<?php namespace GitSwag;
/*
Plugin Name: GitSwag
Description: Display data from your Github profile and repos. This plugin requires PHP 5.3+
Version: 1.2
Requires at least: 3.2
Author: Kenneth Rapp <kennethrapp1@gmail.com>
License: GPLv2 or later, or MIT
*/

require_once("vendor/autoload.php");
require_once("Preflight.php");

class GitSwag extends Preflight{

	function __construct(){

		parent::__construct();

		// register shortcodes
		if(isset($this->Config['shortcodes'])){ 
			foreach($this->Config['shortcodes'] as $shortcode){
				add_shortcode($this->lc_class.'_'.strtolower($shortcode), array($this, 'Shortcode'.ucfirst($shortcode)));
			}
		}

		// if the current user is logged in and has permissions, start the admin 
		if(current_user_can('manage_options')){

			if(is_readable($this->plugin_name.'/Admin.php')){ 

				require_once($this->plugin_name."/Admin.php");										
				
				if($this->admin = new Admin()){ 
					add_action('admin_menu', array($this->admin, 'AdminMenu'));
					add_action('admin_init', array($this->admin, 'AdminInit')); 
				}
			}		
		}
	}

	function ShortcodeReadme($atts, $content){

		if(isset($this->Config['shortcode_readme'])){ 
			
			$shortcode_atts = shortcode_atts($this->Config['shortcode_readme'], $atts);

			$username = $shortcode_atts['github_username'];
			$reponame = $shortcode_atts['github_repo'];
			$readme   = null;

			if(isset($username, $reponame)){
				
				$data_path = dirname(__FILE__)."/data/$username/repos/$reponame/";

				if(is_readable($data_path.'/readme.md')){ 
					$readme = file_get_contents($data_path.'/readme.md');
				}

				else if($readme = $this->GetSomethingFromGithub('repos', $username, $reponame, 'readme')){
					if(isset($readme['encoding']) && ($readme['encoding'] === 'base64')){
						
						$readme = base64_decode($readme['content']);
				
						if(!( is_dir($data_path))) mkdir($data_path, 0700, true);
						
						$this->safe_file_write($data_path.'/readme.md', $readme);
					}
						
				}

				if(isset($readme)){ 
				ob_start();
				
					?>

					<div class="github-readme">

					<?php

					$readme = esc_attr($readme);
					$readme_md = \Parsedown::instance()->parse($readme);
					echo $readme_md;
					
					?>

					</div>

					<?php
					$content = ob_get_contents();
					ob_end_clean();
					return $content;
					
				}
			}
		}
	}

	// shortcode [gitswag_repo_list github_username="username"]
	// display all available repositories as an unordered list
	function ShortcodeRepo_List($atts, $content){

		if(isset($this->Config['shortcode_repo_list'])){ 

			$shortcode_atts = shortcode_atts($this->Config['shortcode_repo_list'], $atts);

			$username = $shortcode_atts['github_username'];

			if(isset($username) && ($repo_data = $this->GetRepoData($username))){
				ob_start();
				?>

				<div class="github-repo">
				<UL class="repolist">

				<?php
				
				foreach ($repo_data as $key => $value) {
					
					$repo = array_filter($repo_data[$key], 'esc_attr');
					
					if((int)$repo['fork'] !== 1){

						?>

						<LI>

							<h2 class="repolist-name"><a href="<?php echo $repo['html_url'];?>"> 
								<?php echo $repo['full_name']; ?></a>
							</h2>

						    <div class="description">
								<?php echo $repo['description']; ?>
							</div>

							<div class="updated-at">
								<strong> <?php echo $repo['language']; ?> </strong>
							  	created: <?php echo date ("m/d/Y", strtotime($repo['created_at'])); ?>,
							  	updated: <?php echo date ("m/d/Y", strtotime($repo['updated_at'])); ?>
							</div>

						</LI>

						<?php
					}
					
				}
				
				?>

				</UL></div>

				<?php
				$content=ob_get_contents();
				ob_end_clean();
				return $content;
			}
		}
	}
	

	// shortcode [gitswag_repo_view github_username="username" github_repo="repo"]
	// display a single repository
	function ShortcodeRepo_View($atts, $content){

		if(isset($this->Config['shortcode_repo_view'])){ 
			
			$shortcode_atts = shortcode_atts($this->Config['shortcode_repo_view'], $atts);

			$username = $shortcode_atts['github_username'];
			$reponame = $shortcode_atts['github_repo'];
			$repo     = null;

			if(isset($username, $reponame)){
				
				$path=dirname(__FILE__)."/data/$username/repos/$reponame/";

				if(is_readable($path."$reponame.json")){

					$repo_dec = json_decode(file_get_contents($path."$reponame.json"), true);
					
					if(json_last_error() === JSON_ERROR_NONE){ 
						$repo = array_filter($repo_dec, 'esc_attr');
					}
				}

				else if($repo_data = $this->GetRepoData($username)){ 

					foreach($repo_data as $key=>$value){
					
						if($repo_data[$key]['full_name'] === "$username/$reponame"){
							
							if(is_dir($path) && (!is_readable($path."$reponame.json"))){ 
								$this->safe_file_write(dirname(__FILE__)."/data/$username/repos/$reponame/$reponame.json", json_encode($repo_data[$key]));
							}

							$repo = array_filter($repo_data[$key], 'esc_attr');
							
						}	
					}
				}

				if(isset($repo)){ 
					ob_start();
					?>
					
					<div class="github-repo" id="<?php echo $repo["full_name"];?>">
						
						<div class="github-header">
							
							<a href="<?php echo esc_attr($repo['owner']['url']); ?>">
								<img border="0" class="github_avatar" src="<?php echo esc_attr( $repo['owner']['avatar_url'] ); ?>">
							</a>

							<a href="<?php echo $repo['html_url']; ?>">
								<?php echo $repo["full_name"]; ?>
							</a>
						</div>

					<?php
					
					if(isset($repo['description'])){

						?>

						<div class="github-description">
							<?php echo $repo['description']; ?>
						</div>

						<?php
					}

					if(isset($repo['language'])){
			
						?>

						<span class="data-item">
							<?php echo " Language: ".$repo['language']; ?>
						</span>

						<?php
					}

					if(isset($repo['forks']) && ($repo['forks']) > 0 ){
						
						?>

						<span class="data-item">
							<a href="<?php echo $repo['url'].'/forks';?>">
								<?php echo " Stars : ".$repo['forks']; ?>
							</a>
						</span>

						<?php
					}

					if(isset($repo['watchers']) && ($repo['watchers']) > 0 ){

						?>

						<span class="data-item">
							<a href="<?php echo $repo['url'].'/watchers';?>">
								<?php echo " Stars : ".$repo['watchers']; ?>
							</a>
						</span>

						<?php
					}

					// show stargazers
					if(isset($repo['stargazers_count']) && ($repo['stargazers_count']) > 0 ){
						
						?>

						<span class="data-item">
							<a href="<?php echo $repo['url'].'/stargazers';?>">
								<?php echo " Stars : ".$repo['stargazers_count']; ?>
							</a>
						</span>

						<?php
					}

					if(isset($repo['open_issues']) && ($repo['open_issues']) > 0 ){
				
						?>

						<span class="data-item">
							<a href="<?php echo $repo['url'].'/issues';?>">
								<?php echo " Open Issues: ".$repo['open_issues']; ?>				
							</a>
						</span>

						<?php
					}
					
					
					?>

					</div>

					<?php
					$content=ob_get_contents();
					ob_end_clean();
					return $content;
				}
			}
		}
	}
	
	function ShortcodeGist_List($atts, $content){

		if(isset($this->Config['shortcode_gist_list'])){ 
			
			$shortcode_atts = shortcode_atts($this->Config['shortcode_gist_list'], $atts);
			
			$username = $shortcode_atts['github_username'];
			
			$gists = null;
	
			if(isset($username)){
				
				$path=dirname(__FILE__)."/data/$username/";
				
				if(is_readable($path."gists.json")){
					
					$gist_dec = json_decode(file_get_contents($path."gists.json"), true);

					if(json_last_error() === JSON_ERROR_NONE){
						$gists = array_filter($gist_dec, 'esc_attr');
					}	
				
				}

				else if($gists = $this->GetSomethingFromGithub('users', $username,'gists')){

					if(!(is_dir($path))) mkdir($path, 0700, true);
					
					$this->safe_file_write($path."gists.json", json_encode($gists));
				}

				if(is_array($gists)){
					ob_start();
					foreach($gists as $key=>$value){
					
						?>

						<UL>
						
						<div>
						<?php echo esc_attr($gists[$key]['description']); ?>
						</div>

						<?php
						
						$files = array_filter($gists[$key]['files'], 'esc_attr');

						foreach($files as $file){ 

							?>

							<LI class="gist-fileheader">	

								<a href="<?php echo $file['raw_url']; ?>">
									<?php echo $file['filename']; ?>
								</a>
						
								Language: 
								<?php echo $file['language']; ?> 
							  ( <?php echo $file['type']; ?>, 
								<?php echo round((int)$file['size']/1024); ?> kb )

							</LI>

							<?php
						}

						?>

						</UL>
						<?php
					}
					$content=ob_get_contents();
					ob_end_clean();
					return $content;
				}
			}
		}
	}

	function ShortcodeGist_View($atts, $content){

		if(isset($this->Config['shortcode_gist_view'])){ 
			
			$shortcode_atts = shortcode_atts($this->Config['shortcode_gist_view'], $atts);
			
			$id = $shortcode_atts['gist_id'];
			
			$gist = null;
	
			if(ctype_digit($id)){

				$path=dirname(__FILE__)."/data/gists/";
			
				if(is_readable($path."$id.json")){
				
					$gist_dec = json_decode(file_get_contents($path."$id.json"), true);

					if(json_last_error() === JSON_ERROR_NONE){
						$gist = $gist_dec;
					}	
				}

				else if($gist = $this->GetSomethingFromGithub('gists', $id)){

					if(!(is_dir($path))) mkdir($path, 0700, true);
					
					$this->safe_file_write($path."$id.json", json_encode($gist));
						
				}

				if(is_array($gist)){
					
					ob_start();
					
					foreach($gist['files'] as $file){
						
						?>
						<div class="gist-fileheader">	

							<a href="<?php echo $file['raw_url']; ?>">
								<?php echo $file['filename']; ?>
							</a>
					
							Language: 
							<?php echo $file['language']; ?> 
						  ( <?php echo $file['type']; ?>, 
							<?php echo round((int)$file['size']/1024); ?> kb )

						</div>
						<div class="file-preview">
						<pre class="prettyprint">
						<?php 
						// improve legibility - replace tabs with 2 spaces
						$content = trim(preg_replace('/\t/', '  ', $file['content']));
						echo esc_attr($content); 
						?>
						</pre></div>
						<?php
					}
					
					$content=ob_get_contents();
					ob_end_clean();
					return $content;
				}
			}
		}
	}

// get the latest commit message: http://stackoverflow.com/questions/15919635/on-github-api-what-is-the-best-way-to-get-the-last-commit-message-associated-w
	function ShortcodeFile_View($atts, $content){

		if(isset($this->Config['shortcode_file_view'])){ 
			
			$shortcode_atts = shortcode_atts($this->Config['shortcode_file_view'], $atts);
			
			$username = $shortcode_atts['github_username'];
			$reponame = $shortcode_atts['github_repo'];
			$filepath = $shortcode_atts['path'];
			$filename = $shortcode_atts['filename'];
			$content  = null;
	
			if(isset($username,$reponame,$filepath,$filename)){

				$path = dirname(__FILE__)."/data/$username/repos/$reponame/files/$filepath/";

				$src = $path.$this->slugify($filename).".json";

				if(is_readable($src)){
				
					$content_dec = json_decode(file_get_contents($src), true);

					if(json_last_error() === JSON_ERROR_NONE){
						$content = $content_dec;
					}	
				}

				///repos/:user/:repo/contents/:path
				else if($content = $this->GetSomethingFromGithub('repos', $username, $reponame, 'contents', $filepath.$filename)){
					
					$latest_commit = $this->GetSomethingFromGithub('repos', $username, $reponame, 'commits');
					
					if(!(is_dir($path))) mkdir($path, 0700, true);
					
					$this->safe_file_write($src, json_encode($content));
				
				}

				if(is_array($content)){
					ob_start();
					
					if(isset($content['encoding']) && ($content['encoding'] === 'base64')){
						
						$html_url = $content['html_url'];
						$size     = $content['size'];
						$content  = base64_decode($content['content']);
					
						?>
					

						<div class="file-header">
						
						<h3 class="file-name">
							<a href="https://github.com/<?php echo esc_attr($username); ?>">
							<?php echo esc_attr($username); ?></a> /

							<a href="https://github.com/<?php echo esc_attr($username.'/'.$reponame); ?>">
							<?php echo esc_attr($reponame); ?></a> /
							
							<?php if(!empty($filepath)){ 
								echo esc_attr($filepath).'/'; 
							} ?>
							
							<a href="<?php echo esc_attr($html_url); ?>">
								<?php echo esc_attr($filename); ?>
							</a>

							
						</h3>
						
						size: <?php echo round( (int)$size/1024 ).'kb'; ?> 
						
						</div>

						<div class="file-preview">
						<pre class="prettyprint">
		
						<?php 

						// improve legibility - replace tabs with 2 spaces
						$content = trim(preg_replace('/\t/', '  ', $content));
						echo esc_attr($content); 

						?>

						</pre></div>
					
						<?php							
					}
					
					$content=ob_get_contents();
					ob_end_clean();
					return $content;
				}
			}
		}
	}

	function ShortcodeUser_View($atts, $content){

		if(isset($this->Config['shortcode_user_view'])){ 
			
			$shortcode_atts = shortcode_atts($this->Config['shortcode_user_view'], $atts);
			
			$username = $shortcode_atts['github_username'];
			$content  = null;
	
			if(isset($username)){

				$path = dirname(__FILE__)."/data/$username/";

				$src = $path."$username.json";

				if(is_readable($src)){
				
					$content_dec = json_decode(file_get_contents($src), true);

					if(json_last_error() === JSON_ERROR_NONE){
						$content = $content_dec;
					}	
				}

				///repos/:user/:repo/contents/:path
				else if($content = $this->GetSomethingFromGithub('users', $username)){
					
					if(!(is_dir($path))) mkdir($path, 0700, true);

					$this->safe_file_write($src, json_encode($content));
				}


				if(isset($content)){

					$content = array_filter($content, 'esc_attr');
					ob_start();
					

					?>
					
					<div class="column-sec vcard" itemtype="http://schema.org/Person">
					
					<h4 class="vcard-names">
					<img align="left" class="github_avatar" src="<?php echo $content['avatar_url'] ?>">	
					<span class="vcard-fullname"><?php echo $content['name'] ?></span>
					<span class="vcard-username"><?php echo $content['login'] ?></span>
					</h4>

					<ul class="vcard-details">
						<LI class="vcard-detail">Location: <?php echo $content['location']; ?></LI>
						<LI class="vcard-detail">Email: <?php echo $content['email']; ?></LI>
						<LI class="vcard-detail">Created: <?php echo date ("m/d/Y", strtotime($content['created_at'])); ?></LI>
						<LI class="vcard-detail">Last Update: <?php echo date ("m/d/Y",strtotime($content['updated_at'])); ?></LI>
					
						<?php

						if(isset($content['company'])) {
							?>
							<LI class="vcard-detail">Company: <?php echo $content['company']; ?></LI>
							<?php
						}

						if(isset($content['blog'])) {
							?>
							<LI class="vcard-detail">Blog: <?php echo $content['blog']; ?></LI>
							<?php
						}

						?>

					</UL>

					<div class="vcard-stats">
						<span class="vcard-stat">
							<strong class="vcard-stat-count">
							<?php echo (int)$content['followers']; ?>
							</strong>
							followers
						</span>
						<span class="vcard-stat">
							<strong class="vcard-stat-count">
							<?php echo (int)$content['following']; ?>
							</strong>
							following
						</span>
						<span class="vcard-stat">
							<strong class="vcard-stat-count">
							<?php echo (int)$content['public_repos']; ?>
							</strong>
							repos
						</span>

						<span class="vcard-stat">
							<strong class="vcard-stat-count">
							<?php echo (int)$content['public_gists']; ?>
							</strong>
							gists
						</span>

					</div>
					</div>
					<?php
					$content=ob_get_contents();
					ob_end_clean();
					return $content;
				}
			}
		}
	}

	function ShortcodeRepo_Header($atts, $content){

		if(isset($this->Config['shortcode_repo_header'])){ 

			$shortcode_atts = shortcode_atts($this->Config['shortcode_repo_header'], $atts);

			$username = $shortcode_atts['github_username'];
			$reponame = $shortcode_atts['github_repo'];

			if(isset($username, $reponame) && ($repo_data = $this->GetRepoData($username))){

				foreach ($repo_data as $key => $value) {

					if($repo_data[$key]['full_name'] === "$username/$reponame"){ 
						
						ob_start();
						
						?>

						<div class="github-repo">
						<UL class="repolist">

						<?php
											
						$repo = array_filter($repo_data[$key], 'esc_attr');

						if((int)$repo['fork'] !== 1){

							?>

							<LI>

								<h2 class="repolist-name"><a href="<?php echo $repo['html_url'];?>"> 
									<?php echo $repo['full_name']; ?></a>
								</h2>

							    <div class="description">
									<?php echo $repo['description']; ?>
								</div>

								<div class="updated-at">
									<strong> <?php echo $repo['language']; ?> </strong>
								  	created: <?php echo date ("m/d/Y", strtotime($repo['created_at'])); ?>,
								  	updated: <?php echo date ("m/d/Y", strtotime($repo['updated_at'])); ?>
								</div>

							</LI>

							</UL></div>

							<?php
						}
					}
					$content=ob_get_contents();
					ob_end_clean();
					return $content;
				}
			}
		}
	}


	// return data from either the locally cached json file for a repository or
	// fetch and build that json file first, then return the data (or null)
	// should probably deprecate this.
	function GetRepoData($username, $reponame=null){

		if(is_array($this->repo_data)) return $this->repo_data;

		$data_path = dirname(__FILE__).'/data/'.$username.'/repos/';
		$file = $data_path.$username.'.json';

		// if the cache exists, use it
		if(is_readable(realpath($file))){
			
			$repo_dec = json_decode(file_get_contents($file), true);			
			
			if(json_last_error() === JSON_ERROR_NONE){
				$this->repo_data = $repo_dec;	
				return $this->repo_data;
			}
		}

		else{
			if($repo_data = $this->GetSomethingFromGithub('users', $username, 'repos')){ 
				if(!( is_dir($data_path))) mkdir($data_path, 0700, true);
				$this->safe_file_write($file, json_encode($repo_data));
				$this->repo_data = $repo_dec;
				return $this->repo_data;
				
			}
		}

		return false;
	}

	// generic "get api from github with curl" thing
	private function GetSomethingFromGithub(){

		$path = implode('/', func_get_args());	
		
		$result = $this->GetCurl("https://api.github.com/$path");

		if($result['info']['http_code'] === 200 ){ 

		/* handle error message */

			$result_dec = json_decode($result['content'], true);

			if(json_last_error() === JSON_ERROR_NONE){
				return $result_dec;
			}
		}
		
		$this->safe_file_write($this->plugin_name."/data/.hterrlog", json_encode($content));
		return false;	
	}

}

new GitSwag();