<?php namespace GitSwag;

class PluginUtilities{
	
	function write_ini_file($file, array $array){

		if(current_user_can('manage_options')){ 
	    
		    $res = array();
		    
		    foreach($array as $key => $val){
		        if(is_array($val)){
		            $res[] = "\r\n[$key]";
		            foreach($val as $skey => $sval) $res[] = "$skey = ".(is_numeric($sval) ? $sval : '"'.$sval.'"');
		        }
		        else{
		        	$res[] = "$key = ".(is_numeric($val) ? $val : '"'.$val.'"');
		    	}
		    }
		    
		    if(count($res)){ 
	    		$this->safe_file_write($file, implode("\r\n", $res));
	    	}
	    }
	}

	function safe_file_write($fileName, $dataToSave){    

		if ($fp = fopen($fileName, 'w')){

        	$startTime = microtime();
        
	        do{            
	        	$canWrite = flock($fp, LOCK_EX);
	           // If lock not obtained sleep for 0 - 100 milliseconds, to avoid collision and CPU load
	           if(!$canWrite) usleep(round(rand(0, 100)*1000));
	        
	        } while ((!$canWrite)and((microtime()-$startTime) < 1000));

        //file was locked so now we can store information
	        if ($canWrite){            
	        	fwrite($fp, $dataToSave);
	            flock($fp, LOCK_UN);
	        }
        
        	fclose($fp);
    	}
    }

     function slugify($text){ 
	  // replace non letter or digits by -
	  $text = preg_replace('~[^\\pL\d]+~u', '-', $text);

	  // trim
	  $text = trim($text, '-');

	  // transliterate
	  $text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);

	  // lowercase
	  $text = strtolower($text);

	  // remove unwanted characters
	  $text = preg_replace('~[^-\w]+~', '', $text);

	  if (empty($text)){
	    return 'n-a';
	  }

	  return $text;
	}

	function GetCurl($url){
		
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); 
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311 Firefox/2.0.0.13');
		
		$content = curl_exec($ch);

		$info = curl_getinfo($ch);
		
		curl_close($ch);
		
		return array('info'=>$info, 'content'=>$content);	
	}
}

new PluginUtilities();