<?php namespace GitSwag;

require_once(ABSPATH.'wp-includes/pluggable.php');
require_once(ABSPATH.'wp-includes/formatting.php');
require_once(ABSPATH.'wp-admin/includes/plugin.php');


/* 	checks for the minimum required php and wordpress versions,
	loads general plugin data to be referred to by child classes
	and preloads necessary wp files.  */

require_once("PluginUtilities.php");

class Preflight extends PluginUtilities{
	
	private $Settings = array();

	// global write-once setter and getter. Any attribute assigned as $this
	// by a class extending Preflight can't then be reassigned. 

	function __set($key, $value){
		if(!array_key_exists($key, $this->Settings)){
			$this->Settings[$key] = $value;
		}
	}

	function __get($key){
		if(array_key_exists($key, $this->Settings)){
			return $this->Settings[$key];
		}
		return null;
	}

	function __construct(){

		global $wp_version;
		
		// the name of the plugin = the name of the directory
		$this->plugin_name = dirname(__FILE__);
		
		// initial setting keys, shortcode names and stuff...
		$this->Config = parse_ini_file(($this->plugin_name.'/.htplugincfg'), true);

		// the name of the primary file = the namespace of the plugin
		$this->plugin_file = $this->plugin_name.'/'.__NAMESPACE__.'.php';

		// prefetch wordpress plugin data
		$this->plugin_data = get_plugin_data($this->plugin_file);


		// lowercase and uc-first versions of the canonical plugin name for reference 
		$this->uc_class = ucfirst(preg_replace("/[^\w\d ]/ui",    '', $this->plugin_data['Name']));
		$this->lc_class = strtolower(preg_replace("/[^\w\d ]/ui", '', $this->plugin_data['Name']));

		// get and save the user info
		$this->userinfo = get_userdata(get_current_user_id()); 

		// settings for this plugin will automatically be named as the
		// lc name of the plugin + _options. 
		$this->settings_group = $this->lc_class.'_options';

		// die and deactivate if php isn't up to snuff
		if (version_compare( PHP_VERSION, $this->Config['required']['php'], '<' )) {
			deactivate_plugins( $this->plugin_file );
			wp_die( wp_sprintf('Sorry, %1s requires PHP %2s+. Your server is currently running PHP %3s. This plugin has been deactivated.', $this->plugin_data["Name"] , GS_REQUIRED_PHP, PHP_VERSION));		
		}

		// die and deactivate if wordpress isn't up to snuff. 
		if (version_compare( $wp_version, $this->Config['required']['wordpress'], '<' )) {
			deactivate_plugins( $this->plugin_file );
			wp_die( wp_sprintf('Sorry, %1s requires Wordpress %2s+. Your server is currently running Wordpress %3s. This plugin has been deactivated.', $this->plugin_data["Name"] , GS_REQUIRED_WP, $wp_version));		
		}

		// enqueue scripts
		if(isset($this->Config['scripts'])){
			foreach($this->Config['scripts'] as $js_uri){ 	
				wp_register_script("js_".$this->lc_class, $js_uri);
				wp_enqueue_script('js_'.$this->lc_class);
			}
		}

		// enqueue stylesheets
		if(isset($this->Config['stylesheets'])){
			foreach($this->Config['stylesheets'] as $css){ 
				$css_uri = plugins_url('css/'.$css, __FILE__ );
				wp_register_style("css_".$this->lc_class, $css_uri);
				wp_enqueue_style('css_'.$this->lc_class);
			}
		}
	}
}